package robo;
import java.util.*;
import static robo.Constant.*;
public class Robo {
    final private String robot_name;
    final private String robot_colour;
    public Robo(String name, String colour){
	initMsgToRobot();
	robot_name = name;
	robot_colour = colour;
	quitting = false;
	this.initValue();
	robot_option( SEND_ROTATION_REACHED, 1 );
    }

  // These functions are equivalent to messages sent from the robot
    final protected void initValue(){
	robot_rotate = 0.53;
	acceleration = 0.54;
	brake_value = 0.0;
    }
    private <T> void output(String arg0, T arg1){
	System.out.println(arg0+" "+arg1);
    }
    private  <T1, T2> void output(String arg0, T1 arg1, T2 arg2){
	System.out.println(arg0+" "+arg1+" "+arg2);
    }
    private <T1, T2, T3> void
        output(String arg0, T1 arg1, T2 arg2, T3 arg3){
	System.out.println(arg0+" "+arg1+" "+arg2+" "+arg3);
    }
    private  <T1, T2, T3, T4> void
        output(String arg0, T1 arg1, T2 arg2, T3 arg3, T4 arg4){
	System.out.println(arg0+" "+arg1+" "+arg2+" "+arg3+" "+arg4);
    }
    final public void robot_option(int option, int value ){
	output("RobotOption",option,value);
    }
    final protected void name(String name ){
	output("Name",name);
    }
    final protected void colour( String home, String away ){
	output("Colour",home,away);
    }
    final protected void rotate( int what, double vel ){
	output("Rotate",what,vel);
    }
    final protected void rotate_to(int what, double vel, double angle ){
	output("RotateTo",what,vel,angle);
    }
    final protected void rotate_amount(int what, double vel, double angle ){
	output("RotateAmount",what,vel,angle);
    }
    final protected void sweep(int what, double vel, double left,
			       double right ){
	output("Sweep",what,vel,left,right);
    }
    final protected void accelerate(double amount ){
	output("Accelerate",amount);
    }
    final protected void brake(double amount ){
	output("Brake",amount);
    }
    final protected void shoot(double energy ){
	output("Shoot",energy);
    }
    final protected void print(String message ){
	output("Print",message);
    }
    protected void debug(String message ){
	output("Debug", message);
    }
    final protected void debug_line(double start_angle,double start_radius,
			      double end_angle, double end_radius ){
	output("DebugLine",start_angle,start_radius,end_angle,end_radius);
    }
    final protected void debug_circle(double center_angle, double center_radius,
                                double circle_radius ){
	output("DebugCircle",center_angle,center_radius,circle_radius);
    }
    final protected void initialize(int first_seq ){
	if( first_seq == 1 ){
	    name( robot_name );
	    colour( robot_colour, robot_colour );
	}
    }
    final protected void game_option( int option, double value ){
	switch( option ){
	case ROBOT_MAX_ROTATE:
	    robot_max_rotate = value;
	    break;
	case ROBOT_CANNON_MAX_ROTATE:
	    robot_cannon_max_rotate = value;
	    break;
	case ROBOT_RADAR_MAX_ROTATE:
	    robot_radar_max_rotate = value;
	    break;
	case ROBOT_MAX_ACCELERATION:
	    robot_max_acceleration = value;
	    break;
	case ROBOT_MIN_ACCELERATION:
	    robot_min_acceleration = value;
	    break;
	case ROBOT_START_ENERGY:
	    robot_start_energy = value;
	    break;
	case ROBOT_MAX_ENERGY:
	    robot_max_energy = value;
	    break;
	case ROBOT_ENERGY_LEVELS:
	    robot_energy_levels = value;
	    break;
	case SHOT_SPEED:
	    shot_speed = value;
	    break;
	case SHOT_MIN_ENERGY:
	    shot_min_energy = value;
	    break;
	case SHOT_MAX_ENERGY:
	    shot_max_energy = value;
	    break;
	case SHOT_ENERGY_INCREASE_SPEED:
	    shot_energy_increase_speed = value;
	    break;
	case TIMEOUT:
	    timeout = value;
	    break;
	case DEBUG_LEVEL:
	    debug_level = value;
	    break;
	case SEND_ROBOT_COORDINATES:
	    send_robot_coordinates = (int) value;
	    break;
	}
    }
    final protected void pre_checking_messages(){
	if(Math.random()<0.05 && rotate_allowed ){
	    robot_rotate = -robot_rotate;
	    rotate( 1, robot_rotate );
	    if( debug_level >= 2 )
		output("Debug New robot_rotate :",robot_rotate);
	}
    }
    final public void check_messages(){
	quitting = false;
	String msg_name;
	Action act;
	Scanner sc = new Scanner(System.in);
	while( sc.hasNext() && !quitting ){
	    msg_name = sc.next();
	    act = msgToRobot.get(msg_name);
	    if(act!=null){
		act.perform(sc);
	    }else{
		print("unknown message: "+msg_name);
	    }
	}
    }
    protected void warning                (int type, String message ){
	String comment="";
	switch( type ) {
	case UNKNOWN_MESSAGE:
	    comment = "Unknown message: ";
	    break;
	case PROCESS_TIME_LOW:
	    comment = "Process time low: ";
	    break;
	case MESSAGE_SENT_IN_ILLEGAL_STATE:
	    comment = "Message sent in illegal state: ";
	    break;
	case UNKNOWN_OPTION:
	    comment = "Unknown option: ";
	    break;
	case OBSOLETE_KEYWORD:
	    comment = "Obsolete Keyword: ";
	    break;
	}
	print( comment+message );
    }
    final protected void exit_robot(){
	print("Shutting down and leaving");
	quitting = true;
    }
    final protected boolean is_quitting () { return quitting; }
    protected boolean quitting;
    // Boolean variables to control if it is allowed to change robot
    // rotation and acceleration.
    protected boolean rotate_allowed;
    protected boolean acceleration_change_allowed;

    
    final private HashMap<String, Action> msgToRobot 
	= new HashMap<String,Action>();
    private void initMsgToRobot(){
	msgToRobot.put("Initialize", new Initialize());
	msgToRobot.put("YourName", new YourName());
	msgToRobot.put("YourColour", new YourColour());
	msgToRobot.put("GameOption", new GameOption());
	msgToRobot.put("GameStarts", new GameStarts());
	msgToRobot.put("Radar", new Radar());
	msgToRobot.put("Info", new Info());
	msgToRobot.put("Coordinates", new Coordinates());
	msgToRobot.put("RobotInfo", new RobotInfo());
	msgToRobot.put("RotationReached", new RotationReached());
	msgToRobot.put("Energy", new Energy());
	msgToRobot.put("RobotsLeft", new RobotsLeft());
	msgToRobot.put("Collision", new Collision());
	msgToRobot.put("Warning", new Warning());
	msgToRobot.put("Dead", new Dead());
	msgToRobot.put("GameFinishes", new GameFinishes());
	msgToRobot.put("ExitRobot", new ExitRobot());
    }
    abstract class Action {
	abstract public void perform(Scanner sc);
    }
    class Initialize extends Action {
	@Override
	    public void perform(Scanner sc){
	    int init = sc.nextInt();
	initialize( init );
	}
    }
    class YourName extends Action {
	@Override
	    public void perform(Scanner sc){
	final String name = sc.next();
	your_name(name);
	}
    }
    class YourColour extends Action {
	@Override
	    public void perform(Scanner sc){
	    final String col = sc.next();
	    your_colour(col);
	}
    }
    class GameOption extends Action {
	@Override
	    public void perform(Scanner sc){
	final int nr = sc.nextInt();
	final double value = sc.nextDouble();
	game_option(nr, value);
	}
    }
    class GameStarts extends Action {
	@Override
	    public void perform(Scanner sc){
	    game_starts();
	}
    }
    class Radar extends Action {
	@Override
	    public void perform(Scanner sc){
	    double dist = sc.nextDouble();
	    int object = sc.nextInt();
	    double angle = sc.nextDouble();
	    switch(object){
	    case NOOBJECT:
		radar_noobject( dist, angle );
		break;
	    case ROBOT:
		radar_robot( dist, angle );
		break;
	    case WALL:
		radar_wall( dist, angle );
		break;
	    case SHOT:
		radar_shot( dist, angle );
		break;
	    case COOKIE:
		radar_cookie( dist, angle );
		break;
	    case MINE:
		radar_mine( dist, angle );
		break;
	    default:
		print("Unknown Object seen!");
		break;
	    }
	}
    }
    class Info extends Action {
	@Override
	    public void perform(Scanner sc){
	final double time = sc.nextDouble();
	final double speed = sc.nextDouble();
	final double canon_angle = sc.nextDouble();
	info( time, speed, canon_angle);
    }
    }
    class Coordinates extends Action {
	@Override
	    public void perform(Scanner sc){
	    final double x = sc.nextDouble();
	    final double y = sc.nextDouble();
	final double rotation = sc.nextDouble();
	coordinates(x,y,rotation);
	}
    }
    class RobotInfo extends Action {
	@Override
	    public void perform(Scanner sc){
	final double energy = sc.nextDouble();
	final int enemy = sc.nextInt();
	robot_info(energy, enemy);
	}
    }
    class RotationReached extends Action {
	@Override
	    public void perform(Scanner sc){
	    final int what = sc.nextInt();
	    rotation_reached(what);
	}
    }
    class Energy extends Action {
	@Override
	    public void perform(Scanner sc){
	    final double en = sc.nextDouble();
	    energy(en);
	}
    }
    class RobotsLeft extends Action {
	@Override
	    public void perform(Scanner sc){
	    final int nr = sc.nextInt();
	    robots_left(nr);
	}
    }
    class Collision extends Action {
	@Override
	    public void perform(Scanner sc){
	    int object = sc.nextInt();
	    double angle = sc.nextDouble();
	    switch(object){
	    case NOOBJECT:
		collision_noobject( angle );
		break;
	    case ROBOT:
		collision_robot( angle );
		break;
	    case WALL:
		collision_wall( angle );
		break;
	    case SHOT:
		collision_shot( angle );
		break;
	    case COOKIE:
		collision_cookie( angle );
		break;
	    case MINE:
		collision_mine( angle );
		break;
	    default:
		print("Collided with Unknown Object!");
		break;
	    }
	}
    }
    class Warning extends Action {
	@Override
	    public void perform(Scanner sc){
	    final int type = sc.nextInt();
	    final String text = sc.nextLine();
	    warning(type,text);
	}
    }
    class Dead extends Action {
	@Override
	    public void perform(Scanner sc){
	    dead();
	}
    }
    class GameFinishes extends Action {
	@Override
	    public void perform(Scanner sc){
	    game_finishes();
	}
    }
    class ExitRobot extends Action {
	@Override
    public void perform(Scanner sc){
	    exit_robot();
	}
    }

    // Override the following method to control your robot
    protected void your_name( String prev_name ){}
    protected void your_colour(String colour ){}
    protected void game_starts            (){}
    protected void radar_noobject (double dist,double angle ){}
    protected void radar_robot    (double dist,double angle ){}
    protected void radar_shot     (double dist,double angle ){}
    protected void radar_wall     (double dist,double angle ){}
    protected void radar_cookie   (double dist,double angle ){}
    protected void radar_mine     (double dist,double angle ){}
    protected void coordinates    (double x,double y,double rotation){}
    protected void info           (double time,double speed,
				   double cannon_angle ){}
    protected void robot_info     (double energy, int enemy ){}
    protected void rotation_reached(int what ){}
    protected void energy          (double energylevel ){}
    protected void robots_left     (int number_of_robots ){}
    protected void collision_noobject     (double angle ){}
    protected void collision_robot        (double angle ){}
    protected void collision_shot         (double angle ){}
    protected void collision_wall         (double angle ){}
    protected void collision_cookie       (double angle ){
	print("Cookie eaten!");
    }
    protected void collision_mine         (double angle ){
	print("Oh no! A mine");
    }
    protected void dead                   (){}
    protected void game_finishes          (){}
    // available the following variables to recognize your robot's condition
    protected  double radar_and_cannon_rotate;
    protected  double robot_rotate;
    protected  double acceleration;
    protected  double brake_value;
    // All game options are stored in these variables.
    protected  double robot_max_rotate;
    protected  double robot_cannon_max_rotate;
    protected  double robot_radar_max_rotate;
    protected  double robot_max_acceleration;
    protected  double robot_min_acceleration;
    protected  double robot_start_energy;
    protected  double robot_max_energy;
    protected  double robot_energy_levels;
    protected  double shot_speed;
    protected  double shot_min_energy;
    protected  double shot_max_energy;
    protected  double shot_energy_increase_speed;
    protected  double timeout;
    protected  double debug_level;
    protected  int send_robot_coordinates;
}



