public class SafeTest
{
public SafeTest(ITest iTest)
{
	m_iTest = iTest;
	m_thrd = null;
}

public static void run(String[] args, String[] TEST_NAMES, ITest[][] TESTS)
{
	boolean bTestInterrupt =
		args.length>0 && args[0].equalsIgnoreCase("-interrupt");

	for (int i=0; i<TESTS.length; ++i)
	{
		ITest iClientTest = TESTS[i][0];
		ITest iServerTest = TESTS[i][1];

		try
		{
			System.out.println("===========================");
			System.out.println(TEST_NAMES[i]);

			SafeTest aServerTest = new SafeTest(iServerTest);
			SafeTest aClientTest = new SafeTest(iClientTest);

			aServerTest.start();
			Thread.sleep(1000);
			aClientTest.start();

			if
			(
				aServerTest.isInterruptibleTest()
				||
				aClientTest.isInterruptibleTest()
			)
			{
				Thread.sleep(1000);
				aServerTest.interrupt();
				aClientTest.interrupt();
			}

			aServerTest.join();
			aClientTest.join();

			aClientTest.dispose();
			aServerTest.dispose();

			System.out.println("===========================");
		}
		catch (Throwable thw)
		{
			thw.printStackTrace();
		}
	}
}

final void start()
{
	if (m_iTest == null)
		return;

	m_thrd = new Thread(m_iTest);
	m_thrd.start();
}

final void join()
{
	if (m_thrd == null)
		return;

	try
	{
		m_thrd.join();
	}
	catch (InterruptedException e)
	{}
}

final void interrupt()
{
	if (m_thrd!=null && m_iTest.isInterruptibleTest())
		m_thrd.interrupt();
}

final void dispose()
{
	if (m_thrd == null)
		return;

	m_iTest.dispose(m_thrd);
}

final boolean isInterruptibleTest()
{
	return m_iTest!=null && m_iTest.isInterruptibleTest();
}

// private //
private Thread m_thrd;
private ITest m_iTest;
}
