/*
 * Copyright (c) 2002 by Mohan Embar. All Rights Reserved
 * May not be reproduced or distributed without permission from the author.
 */ 

package com.thisiscool.util.eclipselauncher.common;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;

public class Controller
{
// Construction
public Controller(IUserInterface iUserInterface, String[] args)
{
	m_iUserInterface = iUserInterface;
	
    // Try opening the configuration file in the current directory
    if (args.length==0)
    {
    	iUserInterface.displayError
    	(
    		"Usage: EclipseLauncher <.elaunch config file>",
    		"Missing Command Line Argument"
    	);
    	System.exit(1);
    }
    readConfigFile(args[0]);
}

// Accessors
public final IUserInterface getUserInterface()
{
	return m_iUserInterface;
}

// Operations
public final void readConfigFile(String strFilename)
{
	IUIList iWorkspaceList = m_iUserInterface.getWorkspaceList();
	iWorkspaceList.clear();
	try
	{
		BufferedReader rdr = new BufferedReader(new FileReader(strFilename));
		m_strEclipseEXE = rdr.readLine();
		m_strEclipseEXE = fixenv(m_strEclipseEXE);
		String strWorkspace;
		while ((strWorkspace = rdr.readLine()) != null)
		{
            int nPos = strWorkspace.indexOf('|');
            if (nPos == -1)
            {
		        m_iUserInterface.displayError
		        (
		        	"Syntax Error: \""+strWorkspace+"\"",
		        	"Syntax Error"
		        );
            }
            else
            {
                String strListName = strWorkspace.substring(0,nPos);
                String strWSDir = strWorkspace.substring(nPos+1);
                m_lstCfgs.add(strListName);
                m_lstDirs.add(strWSDir);
                iWorkspaceList.add(strListName);
            }
		}
	}
	catch (Exception e)
	{}
}

public final void launch(int nSelWorkspaceIndex)
{
	if (nSelWorkspaceIndex == -1)
		return;
	
    String strDir = (String) m_lstDirs.get(nSelWorkspaceIndex);
    StringBuffer sbfCmdLine = new StringBuffer(m_strEclipseEXE+" -data \"");
    sbfCmdLine.append(strDir);
    sbfCmdLine.append('"');

	try
	{
		System.out.println(sbfCmdLine);
		Runtime.getRuntime().exec(sbfCmdLine.toString());
		exit();
	}
	catch (Exception e)
	{
		e.printStackTrace();
		m_iUserInterface.displayError
		(
			e.getClass().getName() + ": " + e.getMessage(),
	        "Failed to Launch Eclipse"
	    );
	}
}

public final void exit()
{
	System.exit(0);
}

// private //
private String m_strEclipseEXE;
private List m_lstCfgs=new ArrayList(), m_lstDirs=new ArrayList();
private IUserInterface m_iUserInterface;

private String fixenv(String str)
{
    int nStartIndex;
    while ((nStartIndex = str.indexOf("%")) != -1)
    {
        int nEndIndex = str.indexOf("%", nStartIndex+1);
        if (nEndIndex == -1)
        {
        	m_iUserInterface.displayError(str, "Syntax Error");
        }

        String strEnvVarName = str.substring(nStartIndex+1, nEndIndex);
        
        String strReplace = null;
        if (strEnvVarName.equals("JAVA_HOME"))
        {
        	strReplace = System.getProperty("java.home");
        }
        if (strReplace == null)
        {
        	m_iUserInterface.displayError
        	(
        		strEnvVarName,
        		"Undefined Environment Variable"
        	);
        }
        str =
        	str.substring(0,nStartIndex) + strReplace + str.substring(nEndIndex+1);
    }
    return str;
}
}
