/*
 * Copyright (c) 2002 by Mohan Embar. All Rights Reserved
 * May not be reproduced or distributed without permission from the author.
 */ 

package com.thisiscool.util.eclipselauncher.swing;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.border.BevelBorder;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.File;

import java.net.URL;

import com.thisiscool.util.eclipselauncher.common.Controller;
import com.thisiscool.util.eclipselauncher.common.IUIList;
import com.thisiscool.util.eclipselauncher.common.IUserInterface;

public class EclipseLauncher extends JFrame
{
public static void main(String[] args)
{
    try
    {
        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch (Exception e)
    {}

    EclipseLauncher aLauncher = new EclipseLauncher(args);
    aLauncher.setVisible(true);
}

// private //
private static final int WIDTH = 300;
private static final int HEIGHT = 250;

private String[] m_arstrArgs;
private JList m_lbWorkspaces;
private DefaultListModel m_lmdlWorkspaces;
private Controller m_Controller;

// private HelpFrame m_HelpFrame;

private EclipseLauncher(String[] args)
{
    super("Eclipse Launcher 0.4");
	m_arstrArgs = args;
    initUI();
}

private void initUI()
{
	// Center it on the screen.
    Dimension dmScreen = Toolkit.getDefaultToolkit().getScreenSize();
    setBounds
    (
        (dmScreen.width >> 1) - (WIDTH >> 1),
        (dmScreen.height >> 1) - (HEIGHT >> 1),
        WIDTH,
        HEIGHT
    );

    initCtrls();
    initController();

    addWindowListener
    (
        new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                onExit();
            }
        }
    );
    
    setIconImage
    (
        Toolkit
            .getDefaultToolkit()
            .getImage
            (
                getClass()
                    .getResource
                    (
                        "/EclipseLauncher.jpg"
                    )
            )
    );
    
}

private void initCtrls()
{
    JPanel pnlAll = new JPanel();
    pnlAll.setLayout(new BorderLayout());
    getContentPane().add(pnlAll);

	JLabel lbl = new JLabel("Workspaces:");
	lbl.setDisplayedMnemonic(KeyEvent.VK_W);
	pnlAll.add(lbl, BorderLayout.NORTH);
	
	m_lbWorkspaces = new JList(m_lmdlWorkspaces = new DefaultListModel());
	pnlAll.add(m_lbWorkspaces, BorderLayout.CENTER);
	lbl.setLabelFor(m_lbWorkspaces);
	m_lbWorkspaces.addMouseListener
	(
		new MouseAdapter()
		{
		public void mouseClicked(MouseEvent e)
		{
			if (e.getClickCount() == 2)
			{
				onBnLaunch();
			}
		}
		}
	);
	m_lbWorkspaces.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
	
	
	JPanel pnlButtons = new JPanel();
	pnlAll.add(pnlButtons, BorderLayout.SOUTH);
    pnlButtons.setLayout(new FlowLayout(FlowLayout.CENTER));
    JButton bn;
    
    pnlButtons.add(bn = new JButton("Launch"));
    bn.setMnemonic(KeyEvent.VK_L);
    bn.addActionListener
    (
        new ActionListener()
        {
        public void actionPerformed(ActionEvent e)
        {
            onBnLaunch();
        }
        }
    );
	
    pnlButtons.add(bn = new JButton("Close"));
    bn.setMnemonic(KeyEvent.VK_C);
    bn.addActionListener
    (
        new ActionListener()
        {
        public void actionPerformed(ActionEvent e)
        {
            onExit();
        }
        }
    );
	
    pnlButtons.add(bn = new JButton("Help"));
    bn.setMnemonic(KeyEvent.VK_H);
    bn.addActionListener
    (
        new ActionListener()
        {
        public void actionPerformed(ActionEvent e)
        {
            onBnHelp();
        }
        }
    );
}

private void initController()
{
	m_Controller = new Controller(new MyUserInterface(), m_arstrArgs);
}

// Message Handlers
private void onExit()
{
    m_Controller.exit();
}

private void onBnLaunch()
{
	int nSelIndex = m_lbWorkspaces.getSelectedIndex();
	m_Controller.launch(nSelIndex);
}

private void onBnHelp()
{
    /*
    if (m_HelpFrame == null)
    {
        URL url = getClass().getResource("/com/thisiscool/util/eclipselauncher/help.html");
        m_HelpFrame = new HelpFrame("Help", url);
    }
    m_HelpFrame.setVisible(true);
    */
}

////////////////////////////////////////////////////////////////
// inner class MyUserInterface

private class MyUserInterface implements IUserInterface
{
public IUIList getWorkspaceList()
{
	return
		new IUIList()
		{
		public final void add(Object o)
		{
			m_lmdlWorkspaces.addElement(o);
		}
		
		public final void clear()
		{
			m_lmdlWorkspaces.removeAllElements();
		}
		};
}

public void displayError(String strMessage, String strTitle)
{
    JOptionPane.showMessageDialog
    (
        EclipseLauncher.this,
        strMessage,
        strTitle,
        JOptionPane.ERROR_MESSAGE
    );
}
}

// end inner class MyUserInterface
////////////////////////////////////////////////////////////////
}
