import java.io.*;
import java.util.*;

public class FileTest
{
public static void main(String[] args)
{
    testEmptyString();
    testListRoots();
    testMkdir();
    testAccess();
    testStat();
    testSetReadOnly();
    testRename();
    testDelete();
    testSetLastModified();
}

// private //
private static final String TEST_ROOT = "/tmp/thisiscool/gcjfiletest";

private static final String TEST_FILE  = TEST_ROOT + "/test.txt";
private static final String TEST2_FILE = TEST_ROOT + "/test2.txt";
private static final String BOGUS_FILE = TEST_ROOT + "/bogus.txt";

private static void testEmptyString()
{
    System.out.println("--> testEmptyString()");
    File f = new File("");
    try
    {
      System.out.println( f.getCanonicalPath() );
    } catch ( Exception e ) {
      System.out.println( e );
    }
    System.out.println();
}

private static void testListRoots()
{
    System.out.println("--> testListRoots()");
    File[] arfil = File.listRoots();
    for (int i=0; i<arfil.length; ++i)
        System.out.println(arfil[i]);
    System.out.println();
}

private static void testMkdir()
{
    System.out.println("--> testMkdir()");
    System.out.println
    (
        "mkdirs "+TEST_ROOT+": " +
            new File(TEST_ROOT).mkdirs()
    );
    System.out.println();
}

private static void testAccess()
{
    System.out.println("--> testAccess()");
    createFile();
    dump(TEST_FILE);
    System.out.println();
}

private static void testStat()
{
    System.out.println("--> testStat()");
    dump(TEST_ROOT);
    dump(TEST_FILE);
    dump(TEST2_FILE);
    System.out.println();
}

private static void testRename()
{
    System.out.println("--> testRename()");
    File fil1 = new File(TEST_FILE);
    File fil2 = new File(TEST2_FILE);
    System.out.println("  renameTo(): "+fil1.renameTo(fil2));
    dump(TEST_ROOT);
    dump(TEST_FILE);
    dump(TEST2_FILE);
    System.out.println();
}

private static void testDelete()
{
    System.out.println("--> testDelete()");
    createFile();
    dump(TEST_FILE);
    File fil = new File(TEST_FILE);
    System.out.println("  delete(): "+fil.delete());
    dump(TEST_ROOT);
    dump(TEST_FILE);
    dump(TEST2_FILE);
    System.out.println();
}

private static void testSetReadOnly()
{
    System.out.println("--> testSetReadOnly()");
    File fil = new File(TEST_FILE);
    System.out.println("  setReadOnly(): "+fil.setReadOnly());
    dump(TEST_FILE);
    System.out.println();
}

private static void testSetLastModified()
{
    System.out.println("--> testSetLastModified()");
    createFile();
    File fil = new File(TEST_FILE);
    fil.setLastModified
    (
		new GregorianCalendar(1995, 5, 16).getTimeInMillis()
	);
    System.out.println();
}

private static void dump(String strFilename)
{
    File fil = new File(strFilename);
    System.out.println("  "+fil+":");
    System.out.println("   exists()     : " + fil.exists());
    System.out.println("   canRead()    : " + fil.canRead());
    System.out.println("   canWrite()   : " + fil.canWrite());
    System.out.println("   isDirectory(): " + fil.isDirectory());
    System.out.println("   isFile()     : " + fil.isFile());
}

private static void createFile()
{
    try
    {
        PrintWriter wrtr =
            new PrintWriter
            (
                new FileWriter(TEST_FILE)
            );
        wrtr.println("Hello from GCJ.");
        wrtr.close();

    }
    catch (Exception e)
    {
        e.printStackTrace();
    }
}

private static Class c1 = gnu.java.locale.Calendar.class;
private static Class c2 = java.util.GregorianCalendar.class;
private static Class c3 = gnu.gcj.convert.Input_ASCII.class;
private static Class c4 = gnu.gcj.convert.Input_UTF8.class;
private static Class c5 = gnu.gcj.convert.Input_8859_1.class;
private static Class c6 = gnu.java.locale.LocaleInformation.class;
private static Class c7 = gnu.gcj.convert.Output_ASCII.class;
}
