public class InterruptTest
{
public static void main(String[] args)
{
	SafeTest.run(args, TEST_NAMES, TESTS);
}

// private //
private static String[] TEST_NAMES =
{
	"simple test",
	"Process.waitFor(), normal termination",
	"Process.waitFor(), interrupt",
	"Thread.sleep(), normal termination",
	"Thread.sleep(), interrupt",
	"infinite wait(), notify()",
	"infinite wait(), interrupt()",
	"finite wait(), notify()",
	"finite wait(), timeout(), condition met",
	"finite wait(), interrupt()",
	"infinite wait(), notifyAll()",
	"infinite wait(), interrupt one, notifyAll()",
	"infinite wait(), interrupt all",
};

private static ITest[][] TESTS =
{
	{
		new SimpleTest("TrivialClient"),
		new SimpleTest("TrivialServer")
	},
	{
		null,
		new HangTest(false)
	},
	{
		null,
		new HangTest(true)
	},
	{
		null,
		new SleepTest(false)
	},
	{
		null,
		new SleepTest(true)
	},
	{
		new WakeupTest(0, WakeupTest.NOTIFY_ONE),
		new WaitTest(0, false)
	},
	{
		null,
		new WaitTest(0, true)
	},
	{
		new WakeupTest(0, WakeupTest.NOTIFY_ONE),
		new WaitTest(1000, false)
	},
	{
		new WakeupTest(0, WakeupTest.NO_NOTIFY),
		new WaitTest(3000, false)
	},
	{
		null,
		new WaitTest(5000, true)
	},
	{
		new WakeupTest(0, WakeupTest.NOTIFY_ALL),
		new MultiWaitTest(0, MultiWaitTest.INTERRUPT_NONE, false)
	},
	{
		null,
		new MultiWaitTest(0, MultiWaitTest.INTERRUPT_ONE, true)
	},
	{
		null,
		new MultiWaitTest(0, MultiWaitTest.INTERRUPT_ALL, true)
	},
	
//	TODO: interrupt just one, notify others
};

private static ITest m_iCurrentServerTest = null;

//////////////////////////////////////////////////////
// nested class SimpleTest

private static class SimpleTest extends TestBase
{
// Runnable overrides
public final void run()
{
	out().println("Sleeping for 1s");
try
	{
		Thread.sleep(1000);
		out().println("That was WONDERFUL!");
	}
	catch (InterruptedException e)
	{}
}

// package //
SimpleTest(String strName)
{
	super(strName, false);
}
}

// end nested class SimpleTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class HangTest

private static class HangTest extends TestBase
{
// Runnable overrides
public final void run()
{
	out().println("Launching hang.exe...");
	try
	{
		String strEXEName =
			System.getProperty("os.name").indexOf("Windows") != -1
			?
			"hang_win32.exe"
			:
			"./hang_posix.exe";
		Process prc = Runtime.getRuntime().exec(strEXEName);
		prc.waitFor();
		out().println("hang.exe terminated normally.");
	}
	catch (Exception e)
	{
		dump(e);
	}
}

// package //
HangTest(boolean bInterruptible)
{
	super("Process.waitFor()", bInterruptible);
}
}

// end nested class HangTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class SleepTest

private static class SleepTest extends TestBase
{
// Runnable overrides
public final void run()
{
	out().println("Sleeping for 5s...");
	try
	{
		Thread.sleep(5000);
		out().println("I awoke normally.");
	}
	catch (Exception e)
	{
		dump(e);
	}
}

// package //
SleepTest(boolean bInterruptible)
{
	super("Thread.sleep()", bInterruptible);
}
}

// end nested class SleepTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class WakeupTest

private static class WakeupTest extends TestBase
{
// Runnable overrides
public final void run()
{
	try
	{
		synchronized (m_iCurrentServerTest)
		{
			if (m_nSleepTime > 0)
				Thread.sleep(m_nSleepTime);
			switch (m_nNotify)
			{
				case NOTIFY_ONE:
					m_iCurrentServerTest.notify();
					break;
				case NOTIFY_ALL:
					m_iCurrentServerTest.notifyAll();
					break;
			}
			if (m_iCurrentServerTest instanceof WaitTest)
				((WaitTest) m_iCurrentServerTest).m_bCondition = true;
			else
				((MultiWaitTest) m_iCurrentServerTest).m_bCondition = true;
		}
	}
	catch (InterruptedException e)
	{}
}

// package //
final static int NO_NOTIFY = 0;
final static int NOTIFY_ONE = 1;
final static int NOTIFY_ALL = 2;

WakeupTest(int nSleepTime, int nNotify)
{
	super("Wakeup Test", false);
	m_nSleepTime = nSleepTime;
	m_nNotify = nNotify;
}

// private //
private int m_nSleepTime;
private int m_nNotify;
}

// end nested class WaitTest
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
// nested class WaitTest

private static class WaitTest extends TestBase
{
// Runnable overrides
public final void run()
{
	out().println("I am getting sleepier....");
	m_iCurrentServerTest = this;
	try
	{
		synchronized (this)
		{
			while (!m_bCondition)
			{
				if (m_nTimeout > 0)
					wait(m_nTimeout);
				else
					wait();
			}
			out().println("What a strange dream!");
		}
	}
	catch (Exception e)
	{
		dump(e);
	}
}

// package //
WaitTest(int nTimeout, boolean bInterruptible)
{
	super("Wait test", bInterruptible);
	m_nTimeout = nTimeout;
}

// private //
private int m_nTimeout;
private boolean m_bCondition = false;
}

// end nested class WaitTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class MultiWaitTest

private static class MultiWaitTest extends TestBase
{
// Runnable overrides
public final void run()
{
	out().println("They are getting sleepier....");
	m_iCurrentServerTest = this;
	try
	{
		Thread[] arthrd = new Thread[10];
		for (int i=0; i<arthrd.length; ++i)
		{
			Thread thrd =
				new Thread
				(
					new Runnable()
					{
					public final void run()
					{
						String strMyName =
							Thread
								.currentThread()
								.getName();
						try
						{
							synchronized (m_iCurrentServerTest)
							{
								while (!m_bCondition)
								{
									if (m_nTimeout > 0)
										m_iCurrentServerTest.wait(m_nTimeout);
									else
										m_iCurrentServerTest.wait();
								}
								out().println
								(
									"I'm awake."
								);
							}
						}
						catch (InterruptedException e)
						{
							out().println
							(
								"Interrupted!"
							);
						}
					}
					}
					,
					"Thread "+i
				);
			arthrd[i] = thrd;
			thrd.start();
		}
		
		// Wait for the threads to fire up
		Thread.sleep(1000);
		
		// Now wait for all of the threads
		boolean bInterrupted = false;
		for (int i=0; !bInterrupted && i<arthrd.length; ++i)
		{
			try
			{
				arthrd[i].join();
			}
			catch (InterruptedException e)
			{
				bInterrupted = true;
			}
		}
		
		if (bInterrupted)
		{
			switch (m_nInterrupt)
			{
				case INTERRUPT_ONE:
				{
					synchronized (this)
					{
						try
						{
							arthrd[3].interrupt();
						}
						catch (Throwable thw)
						{}
					}
					
					Thread.sleep(500);
					
					synchronized (this)
					{
						notifyAll();
						m_bCondition = true;
							// wake up the rest normally
					}
					break;
				}
				case INTERRUPT_ALL:
				{
					for (int i=0; i<arthrd.length; ++i)
					{
						try
						{
							arthrd[i].interrupt();
						}
						catch (Throwable thw)
						{}
					}
					break;
				}
			}

			for (int i=0; i<arthrd.length; ++i)
			{
				try
				{
					arthrd[i].join();
				}
				catch (Throwable thw)
				{}
			}
		}
	}
	catch (Exception e)
	{
		dump(e);
		e.printStackTrace();
	}
}

// package //
static final int INTERRUPT_NONE = 0;
static final int INTERRUPT_ONE = 1;
static final int INTERRUPT_ALL = 2;

MultiWaitTest(int nTimeout, int nInterrupt, boolean bInterruptible)
{
	super("Multi Wait test", bInterruptible);
	m_nTimeout = nTimeout;
	m_nInterrupt = nInterrupt;
}

// private //
private int m_nTimeout;
private boolean m_bCondition = false;
private int m_nInterrupt;
}

// end nested class MultiWaitTest
//////////////////////////////////////////////////////

}
