/* ThisIsCool Nim Copyright (C) 2003  Mohan Embar

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

ThisIsCool Nim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ThisIsCool Nim; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

/* Written by Mohan Embar <gnustuff@thisiscool.com>. */

package com.thisiscool.nim;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

///////////////////////////////////////////////////////////////////////
// class MessageBox

/**
 * A simple AWT message box.
 */
public class MessageBox extends Dialog
{
/**
 * Button constants
 */
public static final int YES    = 1;
public static final int NO     = 2;
public static final int OK     = 4;
public static final int CANCEL = 8;

/**
 * Icon constants
 */
public static final int NO_ICON     = 0;
public static final int STOP        = 1;
public static final int QUESTION    = 2;
public static final int EXCLAMATION = 3;
public static final int SMILE       = 4;

/**
 * Displays a message with a title.
 * @param   frmParent           My parent frame or <code>null</code>
 * @param   strMsg              The message to display
 * @param   strTitle            The title of the message box. Can be <PRE>null</PRE>
 * @param   nButtons            The buttons in this box. The above button constants can be ORed.
 * @param   nIcon               One of the above icon constant values
 * @return
 *      the button constant value of the button pressed to dismiss
 *      the dialog box.
 */
public static int display
(
	Frame frmParent,
	String strMsg,
	String strTitle,
	int nButtons,
	int nIcon
)
{
	if (strTitle == null)
		strTitle = new String();

	if (frmParent == null)
		frmParent = new Frame();

	MessageBox dlgMsg =
		new MessageBox
		(
			frmParent,
			strTitle,
			strMsg,
			nButtons,
			nIcon
		);

	dlgMsg.invoke();

	return dlgMsg.m_nButtonPressed;
}

// protected //
/**
 * The button pressed to dismiss the dialog.
 */
protected int m_nButtonPressed;

// package //
MessageBox
(
	Frame frmParent,
	String strTitle,
	String strMsg,
	int nButtons,
	int nIcon
)
{
	super(frmParent, strTitle, true);

    setFont(new Font("Helvetica", Font.PLAIN, 12));
	setLayout(new GridLayout(2,1));
		// The dialog has two rows and one column

	// Add the two panels, setting their LayoutManagers to
	// a centered FlowLayout. Add a Label to the top panel
	// with strMsg and an [OK] button to the bottom panel.
	Panel pnlTop, pnlBottom;
	add(pnlTop = new Panel(new FlowLayout(FlowLayout.CENTER)));

	Label lblMsg;
	if (strMsg != null)
		pnlTop.add(lblMsg = new Label(strMsg, Label.CENTER));

	add(pnlBottom = new Panel(new FlowLayout(FlowLayout.CENTER)));

	// Add as many buttons as we need to. Give them ActionListeners
	// which correctly set m_nButtonPressed.
	// Create our button string
	for (int i=0; i<m_farnButtons.length; ++i)
	{
		int nThisButton = nButtons & m_farnButtons[i];
		if (nThisButton != 0)
		{
			Button bn = new Button(m_farstrButtonText[i]);
			pnlBottom.add(bn);

			final int fnButtonPressed = nThisButton;

			bn.addActionListener
			(
				new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						m_nButtonPressed = fnButtonPressed;
						setVisible(false);
							// Terminate the dialog.
					}
				}
			);
		}
	}

	pack();
}

/**
 * Invokes the dialog in the center of the screen.
 */
final void center(boolean bDisplay)
{
	Rectangle r = getBounds();
	Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
	d.width >>>= 1;
	d.height >>>= 1;
	r.width >>>= 1;
	r.height >>>= 1;
	d.width -= r.width;
	d.height -= r.height;
	setLocation(d.width,d.height);
	if (bDisplay)
		setVisible(true);
}

/**
 * Invokes the box modally
 */
void invoke()
{
	center(true);
}

// private //
private static final int m_farnButtons[] =
{
	YES,
	NO,
	OK,
	CANCEL
};

private static final String m_farstrButtonText[] =
{
	"Yes",
	"No",
	"OK",
	"Cancel"
};
}