/* ThisIsCool Nim Copyright (C) 2003  Mohan Embar

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

ThisIsCool Nim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ThisIsCool Nim; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

/* Written by Mohan Embar <gnustuff@thisiscool.com>. */

package com.thisiscool.nim;

public class NimModel
{
/**
 * Constructor.
 */
public NimModel(int nNbRows)
{
    if (nNbRows<0 || nNbRows>=10)
    {
        throw new RuntimeException("Too many rows: "+nNbRows);
    }

    m_nNbRows = nNbRows;
    int nNbPegs = nNbRows;
    for (int nRow=nNbRows-1; nRow>=0; --nRow)
    {
        m_arnPegs[nRow] = nNbPegs;
        nNbPegs += 2;
    }
}

// Accessors
public final int nbRows()               {return m_nNbRows;}
public final int pegsInRow(int nRow)    {return m_arnPegs[nRow];}

public final boolean done()
{
    int nNbPegsTotal = 0;
    for (int nRow=0; nRow<nbRows(); ++nRow)
        nNbPegsTotal += pegsInRow(nRow);

    return nNbPegsTotal == 0;
}

// Operations
public final boolean makeMove(int nRow, int nNbPegs)
{
    if (nRow>=nbRows() || nNbPegs==0 || pegsInRow(nRow)<nNbPegs)
        return false;

    m_arnPegs[nRow] -= nNbPegs;

    return true;
}

/**
 * Calculate the best move.
 * @param   arnMove
 *      a reference to a preallocated, two-element integer array.
 *      The first element will receive the row to remove the pegs from.
 *      The second element will receive the number of pegs to remove.
 */
public final void calcBestMove(int[] arnMove)
{
    boolean bSolutionFound = false;
    for (int nRow=0; nRow<nbRows() && !bSolutionFound; ++nRow)
    {
        int nXorStart = 0;
        for (int i=0; i<nbRows(); ++i)
        {
            if (i!=nRow)
                nXorStart ^= pegsInRow(i);
        }
        for
        (
            int nNbPegs=1;
            nNbPegs<=pegsInRow(nRow) && !bSolutionFound;
            ++nNbPegs
        )
        {
            int nXor = nXorStart ^ (pegsInRow(nRow)-nNbPegs);
            if (nXor == 0)
            {
                bSolutionFound = true;
                arnMove[0] = nRow;
                arnMove[1] = nNbPegs;
            }
        }
    }

    if (!bSolutionFound)
    {
        int nRowWithMostPegs = 0;
        int nMaxPegs = 0;
        for (int i=0; i<nbRows(); ++i)
        {
            if (pegsInRow(i)>nMaxPegs)
            {
                nMaxPegs = pegsInRow(i);
                nRowWithMostPegs = i;
            }
        }
        arnMove[0] = nRowWithMostPegs;
        arnMove[1] = 1;
    }
}

// private //
private int m_nNbRows;
private int[] m_arnPegs = new int[10];
}
