/* ThisIsCool Nim Copyright (C) 2003  Mohan Embar

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

ThisIsCool Nim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ThisIsCool Nim; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

/* Written by Mohan Embar <gnustuff@thisiscool.com>. */

package com.thisiscool.nim;

import java.awt.*;
import java.awt.event.*;

import java.io.*;

import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * Adapted from <a href="http://www.thisiscool.com/SaveThatCalendar.htm">Save That Calendar!</a>.
 */
public class NimUIGUI
{
public NimUIGUI(Container cnt)
{
    m_cnt = cnt;
    cnt.setFont(new Font("Helvetica", Font.PLAIN, 12));

    init();
        // initialize myself

    layoutUI();
        // lay out our UI components

    updateUI();
}

public final void startGame()
{
    m_Model = new NimModel(3);
    m_Board.repaint();

    int nID =
        MessageBox.display
        (
            m_frmParent,
            "Shall I move first?",
            "First Move",
            MessageBox.YES | MessageBox.NO,
            MessageBox.QUESTION
        );

    if (nID == MessageBox.YES)
        makeComputerMove();

    m_bnMove.setEnabled(true);
}

// private //
/**
 * Our parent frame for child dialogs.
 */
private Frame m_frmParent;

/**
 * My container
 */
private Container m_cnt;

/**
 * Our row and column text fields.
 */
private TextField m_txfRow, m_txfNbPegs;

/**
 * Our status label.
 */
private Label m_lblStatus;

/**
 * Our game board.
 */
private GameBoard m_Board;

/**
 * Our "start" / "move" button
 */
private Button m_bnMove;

/**
 * Our model game board.
 */
private NimModel m_Model = null;

private void init()
{
    Container cntCurrent = m_cnt;
    while (!(cntCurrent instanceof Frame))
        cntCurrent = cntCurrent.getParent();
    m_frmParent = (Frame) cntCurrent;
}

private void layoutUI()
{
    m_cnt.setLayout(new BorderLayout());
    m_cnt.add(m_Board = new GameBoard(), BorderLayout.CENTER);

    Panel pnlBottom = new Panel();
    m_cnt.add(pnlBottom, BorderLayout.SOUTH);
    pnlBottom.setLayout(new GridLayout(4,1));
    pnlBottom.add
    (
        new Label
        (
            "ThisIsCool(TM) Nim by Mohan Embar. (c) 2001.",
             Label.CENTER
        )
    );
    pnlBottom.add
    (
        new Label
        (
            "http://www.thisiscool.com/nim.htm",
             Label.CENTER
        )
    );

    Panel pnlInput = new Panel();
    pnlBottom.add(pnlInput);
    pnlInput.add(new Label("Row:"));
    pnlInput.add(m_txfRow = new TextField(3));
    pnlInput.add(new Label("# Pegs:"));
    pnlInput.add(m_txfNbPegs = new TextField(3));
    m_bnMove = new Button("Move");
    pnlInput.add(m_bnMove);
    m_bnMove.setEnabled(false);
    m_bnMove.addActionListener
    (
        new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                onBnMove();
            }
        }
    );

    Button bn = new Button("New Game");
    pnlInput.add(bn);
    bn.addActionListener
    (
        new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                onBnNewGame();
            }
        }
    );

    pnlBottom.add
    (
        m_lblStatus =
            new Label
            (
                "",
                Label.CENTER
            )
    );

    if (m_cnt instanceof Frame)
    {
        layoutFrame(pnlInput);
        // java.awt.Frame-specific stuff
    }
}

private void layoutFrame(Panel pnlButtons)
{
    final Frame frm = (Frame) m_cnt;

    Button bnExit = new Button("Exit");
    pnlButtons.add(bnExit);
    bnExit.addActionListener
    (
        new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                onFileExit();
            }
        }
    );

    // If we close my frame, exit the app.
    frm.addWindowListener
    (
        new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                onFileExit();
            }
        }
    );

    frm.pack();
}

private void updateUI()
{
}

// Command handlers
private void onBnAbout()
{
}

private void onBnNewGame()
{
    if (m_Model!=null && !m_Model.done())
    {
        int nID =
            MessageBox.display
            (
                m_frmParent,
                "Cancel the game in progress?",
                "Really Quit?",
                MessageBox.YES | MessageBox.NO,
                MessageBox.QUESTION
            );

        if (nID == MessageBox.NO)
            return;
    }

    startGame();
}

private void onBnMove()
{
    m_lblStatus.setText("");
    try
    {
        int nRow = Integer.parseInt(m_txfRow.getText().trim());
        --nRow;

        int nNbPegs = Integer.parseInt(m_txfNbPegs.getText().trim());

        if (m_Model.makeMove(nRow, nNbPegs))
        {
            m_Board.repaint();
            if (m_Model.done())
            {
                MessageBox.display
                (
                    m_frmParent,
                    "I guess you've been eating your bananas lately.",
                    "You win?!",
                    MessageBox.OK,
                    MessageBox.EXCLAMATION
                );
                m_Model = null;
                m_bnMove.setEnabled(false);
                m_Board.repaint();
            }
            else
                makeComputerMove();
        }
        else
        {
            MessageBox.display
            (
                m_frmParent,
                "Illegal move. Try again",
                "No go, buster",
                MessageBox.OK,
                MessageBox.STOP
            );
        }
    }
    catch (Throwable thw)
    {
        String strMsg = thw.getClass().getName()+": "+thw.getMessage();
        m_lblStatus.setText(strMsg);
    }
}

private void onFileExit()
{
    System.exit(0);
}

// Operations
private void makeComputerMove()
{
    if (m_Model.done())
        return;

    int nRow, nNbPegs;
    int[] arnMove = new int[2];
    m_Model.calcBestMove(arnMove);
    nRow = arnMove[0]; nNbPegs = arnMove[1];

    String strMoveMsg =
        "I remove "+nNbPegs+" peg(s) from row "+(nRow+1);
    MessageBox.display
    (
        m_frmParent,
        strMoveMsg,
        "My Move",
        MessageBox.OK,
        MessageBox.SMILE
    );
    m_Model.makeMove(nRow, nNbPegs);
    m_Board.repaint();

    if (m_Model.done())
    {
        MessageBox.display
        (
            m_frmParent,
            "I win. You are an unworthy match for ThisIsCool.",
            "I beat you, sucker!",
            MessageBox.OK,
            MessageBox.SMILE
        );
        m_Model = null;
        m_bnMove.setEnabled(false);
        m_Board.repaint();
    }

    m_txfRow.setText("");
    m_txfNbPegs.setText("");
    m_txfRow.requestFocus();
}

/////////////////////////////////////////////////////////////////////////
// nested class GameBoard

private class GameBoard extends Canvas
{
// Canvas overrides
public void paint(Graphics g)
{
    Dimension dmSize = getSize();
    g.setColor(Color.blue);
    g.fillRect(0, 0, dmSize.width, dmSize.height);

    if (m_Model == null)
        return;

    // Draw the board
    int nY=18;
    for (int nRow=0; nRow<m_Model.nbRows(); ++nRow, nY+=40)
    {
        g.setColor(Color.black);
        g.setFont(m_fnt);
        g.drawString(Integer.toString(nRow+1)+":",8,nY/* If using Sun's AWT: +23 */);
    }

    nY=18;
    for (int nRow=0; nRow<m_Model.nbRows(); ++nRow, nY+=40)
    {
        int nX=40;
        for (int nPeg=0; nPeg<m_Model.pegsInRow(nRow); ++nPeg, nX+=40)
        {
            g.fillOval(nX, nY, 32, 32);
        }
    }
}

public Dimension getPreferredSize()
{
    return getMinimumSize();
}

public synchronized Dimension getMinimumSize()
{
    return m_dmMinSize;
}

// private //
private Font m_fnt = new Font("Helvetica", Font.PLAIN, 24);
private Dimension m_dmMinSize = new Dimension(200,150);

private GameBoard()
{}
}

// end nested class GameBoard
/////////////////////////////////////////////////////////////////////////
}
